// Game.cpp: implementation of the CGame class.
//
//////////////////////////////////////////////////////////////////////

#include "Game.h"
#include "math.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CGame::CGame()
{
	game_over=0; help=0;
	light=1; blend=0; fog=0;
	rot.x=rot.y=rot.z=pos.x=pos.y=pos.z=0; pos.y=30; pos.z=100; rychlost=1.0;
	memset(objekty,0,sizeof(void *)*pocetobj);	// Set The Pointer To NULL
	aktivni_planeta=0;
}

CGame::~CGame()
{
	delete ves;
	for(int i=1/*kvuli vesmiru*/; i<pocetobj; i++)
		delete objekty[i];
	SDL_ShowCursor(1);
	SDL_WM_GrabInput(SDL_GRAB_OFF);
	SDL_Quit();		// Shutdown all subsystems
}

int CGame::InitGame()
{
	if( SDL_Init( SDL_INIT_VIDEO | SDL_INIT_TIMER ) ){
		fprintf( stderr, "Could not initialise SDL: %s\n", SDL_GetError() );
		return 0;
	}
	atexit(SDL_Quit);

	SDL_WM_GrabInput(SDL_GRAB_ON);
	SDL_ShowCursor(0);
	SDL_WarpMouse(0, 0);

	//nacita rozliseni atd. ze souboru "configure.txt"
	int  width=640, height=480, bpp=0, fullscreen=0, oglw=0, oglh=0,
		flags=SDL_OPENGL/*BLIT/*|SDL_HWSURFACE*/;
	FILE *File=NULL;
	char s[1000];
	File=fopen("configure.txt","r");
	if (File) {
		while (fscanf(File,"%s",s)!=EOF) {
			if (strcmp(s,"width")==0) fscanf(File," %d",&width);
			if (strcmp(s,"height")==0) fscanf(File," %d",&height);
			if (strcmp(s,"oglw")==0) fscanf(File," %d",&oglw);
			if (strcmp(s,"oglh")==0) fscanf(File," %d",&oglh);
			if (strcmp(s,"fullscreen")==0) { fscanf(File," %d",&fullscreen); if (fullscreen) flags|=SDL_FULLSCREEN; }
			if (strcmp(s,"fullscreen")==0) { fscanf(File," %d",&fullscreen); if (fullscreen) flags|=SDL_FULLSCREEN; }
			if (strcmp(s,"bpp")==0) fscanf(File," %d",&bpp);
			if (s[0]=='/' && s[1]=='/') while (getc(File)!='\n');
		}
		fclose(File);
	}

	SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
	SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
	SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
	SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 16 );
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
	if ( !(m_screen=SDL_SetVideoMode( width, height, bpp, flags )) ) {
		fprintf(stderr, "Couldn't set GL video mode: %s\n", SDL_GetError());
		SDL_Quit();
		return 0;
	}

	if (oglw==0) oglw=width; if (oglh==0) oglh=height;
	ogl.ReSizeGLScene( oglw, oglh);
	if (!ogl.InitGL()) {		// Initialize Our Newly Created GL Window
		MessageBox(NULL,"Initialization of OGL Failed.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return 0;
	}
	ogl.lighting(light); ogl.blending(blend);

	//loaduje objekty
	for (int i=0; i<pocetobj; i++) {
		objekty[i]=NULL;
		switch (i) {
		case 0: objekty[i] = ves = new CVesmir; break;	//Vesmir
		default: objekty[i] = new CBaseObject; break;
		}
		if ( !objekty[i]) {
			MessageBox(NULL,"Nepodarilo se vytvorit objekt","ERROR",MB_OK|MB_ICONEXCLAMATION);
			return 0;
		}
		if ( !(objekty[i]->InitObject(&ogl)) ) {
			MessageBox(NULL,"Chyba inicializace objektu","ERROR",MB_OK|MB_ICONEXCLAMATION);
			return 0;
		}
	}
	return 1;
}

#define TICK_INTERVAL 10

// main game loop
int CGame::PlayGame()
{
	int disp=1;
	Uint32 next_time, fps=0, min_fps=0;
	char caption[50];
	
	next_time = SDL_GetTicks();
	while (!game_over) {
		while ( SDL_GetTicks() > next_time ) {
			UpdateGameLogic();
			next_time+=TICK_INTERVAL;
			disp=1;
		}

		// fps
		if ( next_time/1000 != min_fps ) {
			min_fps=next_time/1000;
			sprintf(caption,"fps: %d (%d)", fps, 1000/TICK_INTERVAL/*(TICK_INTERVAL>0)?TICK_INTERVAL:1*/);
			SDL_WM_SetCaption( caption, NULL);
			fps=0;
		}

		if ( disp ) { disp=0; fps++; UpdateDisplay(); }
		//SDL_Delay( next_time-SDL_GetTicks());
	}
	return 0;
}

void CGame::UpdateGameLogic()
{
	static SDL_Event event;
	while( SDL_PollEvent( &event ) ){
		switch( event.type ){
		case SDL_MOUSEMOTION:
			rot.y-=(double)event.motion.xrel/3;	// ??? kdyz tam neni (float), ???
			rot.x-=(double)event.motion.yrel/3;	// ??? tak se trochu seka mys ???
			if (rot.x>90) rot.x=90;
			else if (rot.x<-90) rot.x=-90;
			break;
		case SDL_VIDEORESIZE:
			ogl.ReSizeGLScene(event.resize.w, event.resize.h);
			break;
		case SDL_QUIT:
			game_over=1;
			break;
		}
	}

	keys= SDL_GetKeyState(NULL);
	//hyperrychlost
	if (keys[SDLK_SPACE]) { keys[SDLK_SPACE]=0; rychlost*=10.0f; }
	if (keys[SDLK_LALT]) { keys[SDLK_LALT]=0; rychlost/=10.0f; }
	//dopredu, dozadu, doleva, doprava
	if (keys[SDLK_w]) {
		pos.x -= sin(rot.y*piover180) * rychlost;
		pos.z -= cos(rot.y*piover180) * rychlost;
	}
	if (keys[SDLK_s]) {
		pos.x += sin(rot.y*piover180) * rychlost;
		pos.z += cos(rot.y*piover180) * rychlost;
	}
	if (keys[SDLK_a]) {
		pos.x -= sin((rot.y+90)*piover180) * rychlost;
		pos.z -= cos((rot.y+90)*piover180) * rychlost;
	}
	if (keys[SDLK_d]) {
		pos.x += sin((rot.y+90)*piover180) * rychlost;
		pos.z += cos((rot.y+90)*piover180) * rychlost;
	}
	//otaceni (misto mysi)
	if (keys[SDLK_LEFT ]) rot.y+=0.8f;
	if (keys[SDLK_RIGHT]) rot.y-=0.8f;
	if (keys[SDLK_UP]) {
		rot.x+=0.8;
		if (rot.x>90) rot.x=90;
	}
	if (keys[SDLK_DOWN]) {
		rot.x-=0.8;
		if (rot.x<-90) rot.x=-90;
	}
	//vyska
	if (keys[SDLK_LSHIFT]) pos.y+= 1 * rychlost;// if (pos.y>0) pos.y-=0.02f;
	if (keys[ SDLK_LCTRL]) pos.y-= 1 * rychlost;// if (pos.y<0) pos.y+=0.02f;
	//konec, help, pruhlednost, svetlo, mlha
	if (keys[SDLK_ESCAPE]) game_over=1;
	if (keys[SDLK_F1]) { keys[SDLK_F1]=0; help = !help; }
	if (keys[SDLK_b]) { keys[SDLK_b]=0; blend = !blend; ogl.blending(blend); }
	if (keys[SDLK_l]) { keys[SDLK_l]=0; light = !light; ogl.lighting(light); }
	//if (keys[SDLK_f]) { keys[SDLK_f]=0; if (++fog>3) fog=0; ogl.fogging(fog); }

	//zmena vztazne soustavy
	if (keys[SDLK_PAGEUP]) {
		keys[SDLK_PAGEUP]=0;
		if (aktivni_planeta!=0) {
			pos.x+=ves->Planety[aktivni_planeta].pol.x;
			pos.y+=ves->Planety[aktivni_planeta].pol.y;
			pos.z+=ves->Planety[aktivni_planeta].pol.z;
		}
		aktivni_planeta++; if (aktivni_planeta>=poc_planet) aktivni_planeta=0;
		if (aktivni_planeta!=0) {
			pos.x-=ves->Planety[aktivni_planeta].pol.x;
			pos.y-=ves->Planety[aktivni_planeta].pol.y;
			pos.z-=ves->Planety[aktivni_planeta].pol.z;
		}
	}
	if (keys[SDLK_PAGEDOWN]) {
		keys[SDLK_PAGEDOWN]=0;
		if (aktivni_planeta!=0) {
			pos.x+=ves->Planety[aktivni_planeta].pol.x;
			pos.y+=ves->Planety[aktivni_planeta].pol.y;
			pos.z+=ves->Planety[aktivni_planeta].pol.z;
		}
		aktivni_planeta--; if (aktivni_planeta<0) aktivni_planeta=poc_planet-1;
		if (aktivni_planeta!=0) {
			pos.x-=ves->Planety[aktivni_planeta].pol.x;
			pos.y-=ves->Planety[aktivni_planeta].pol.y;
			pos.z-=ves->Planety[aktivni_planeta].pol.z;
		}
	}
	if (keys[SDLK_1]) if (1<poc_planet) {
		aktivni_planeta=1; pos.z=4*ves->Planety[aktivni_planeta].r; pos.x=pos.y=rot.x=rot.y=rot.z=0; }
	if (keys[SDLK_2]) if (2<poc_planet) {
		aktivni_planeta=2; pos.z=4*ves->Planety[aktivni_planeta].r; pos.x=pos.y=rot.x=rot.y=rot.z=0; }
	if (keys[SDLK_3]) if (3<poc_planet) {
		aktivni_planeta=3; pos.z=4*ves->Planety[aktivni_planeta].r; pos.x=pos.y=rot.x=rot.y=rot.z=0; }
	if (keys[SDLK_4]) if (4<poc_planet) {
		aktivni_planeta=4; pos.z=4*ves->Planety[aktivni_planeta].r; pos.x=pos.y=rot.x=rot.y=rot.z=0; }
	if (keys[SDLK_5]) if (5<poc_planet) {
		aktivni_planeta=5; pos.z=4*ves->Planety[aktivni_planeta].r; pos.x=pos.y=rot.x=rot.y=rot.z=0; }
	if (keys[SDLK_6]) if (6<poc_planet) {
		aktivni_planeta=6; pos.z=4*ves->Planety[aktivni_planeta].r; pos.x=pos.y=rot.x=rot.y=rot.z=0; }
	if (keys[SDLK_7]) if (7<poc_planet) {
		aktivni_planeta=7; pos.z=4*ves->Planety[aktivni_planeta].r; pos.x=pos.y=rot.x=rot.y=rot.z=0; }
	if (keys[SDLK_8]) if (8<poc_planet) {
		aktivni_planeta=8; pos.z=4*ves->Planety[aktivni_planeta].r; pos.x=pos.y=rot.x=rot.y=rot.z=0; }
	if (keys[SDLK_9]) if (9<poc_planet) {
		aktivni_planeta=9; pos.z=4*ves->Planety[aktivni_planeta].r; pos.x=pos.y=rot.x=rot.y=rot.z=0; }
	if (keys[SDLK_0]) if (10<poc_planet) {
		aktivni_planeta=10; pos.z=4*ves->Planety[aktivni_planeta].r; pos.x=pos.y=rot.x=rot.y=rot.z=0; }

	
	
	for(int i=0; i<pocetobj; i++)
		objekty[i]->UpdateObjLogic(keys);
}

//           ^ y
//         / | \
//           |
//           |     \
//           +----- + x
//         /       /
//     | /
//   z +-
int loggg(double r)//pomocna fce pro vytisknuti rychlosti
{
	r=log10(r);
	if (r>0) return (int)(r+0.1f);
	else return (int)(r-0.1f);
}
void CGame::UpdateDisplay()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear The Screen And The Depth Buffer
	glLoadIdentity();

	if (help) {
		ogl.glPrint2d( -1, 0.9f,1,1,1,"                  Ovladani:");
		ogl.glPrint2d( -1, 0.8f,1,1,1,"");
		ogl.glPrint2d( -1, 0.7f,1,1,1,"                   F1 - zobrazi tuto napovedu");
		ogl.glPrint2d( -1, 0.6f,1,1,1,"                  Esc - konec");
		ogl.glPrint2d( -1, 0.5f,1,1,1,"                sipky - zmena smeru");
		ogl.glPrint2d( -1, 0.4f,1,1,1," a,s,d,w,LShift,LCtrl - zmena polohy");
		ogl.glPrint2d( -1, 0.3f,1,1,1,"        LAlt,mezernik - zmena hyperrychlosti");
		ogl.glPrint2d( -1, 0.2f,1,1,1,"            num+,num- - zmena rychlosti plynuti casu");
		ogl.glPrint2d( -1, 0.1f,1,1,1," PageUp,PageDown,0..9 - zmena vztazne soustavy");
		ogl.glPrint2d( -1, 0.0f,1,1,1,"                    l - zapne/vypne stiny");
		ogl.glPrint2d( -1,-0.1f,1,1,1,"                   F2 - ulozi aktualni vesmiru");
		ogl.glPrint2d( -1,-0.2f,1,1,1,"                   F3 - nacte vesmir");
	} else {
		ogl.glPrint2d( -1, -1,1,1,1,"%d/%d %s", aktivni_planeta,poc_planet-1,ves->Planety[aktivni_planeta].meno); // vypise na 2D obrazovku
		ogl.glPrint2d( 0.65f, -1,1,1,1,"help - F1");
		ogl.glPrint2d( -1, 0.94f,1,1,1,"hyperrychlost: %i",loggg(rychlost)+4);
		//ogl.glPrint2d( -1, 0.0f,1,1,1,"x:%.2Lf",ves->celkemt);
	}

	glRotated( -rot.x, 1, 0, 0);
	glRotated( -rot.y, 0, 1, 0);
	glRotated( -rot.z, 0, 0, 1);
	glTranslated(-pos.x,-pos.y,-pos.z);

 	for (int i=0; i<pocetobj; i++) {
		glPushMatrix();
			switch (i) {
			case 0:
				ves->Planety[0].pol=pos;
				if (aktivni_planeta!=0) {
					ves->Planety[0].pol.x+=ves->Planety[aktivni_planeta].pol.x;
					ves->Planety[0].pol.y+=ves->Planety[aktivni_planeta].pol.y;
					ves->Planety[0].pol.z+=ves->Planety[aktivni_planeta].pol.z;
					glTranslated( -ves->Planety[aktivni_planeta].pol.x, -ves->Planety[aktivni_planeta].pol.y, -ves->Planety[aktivni_planeta].pol.z);
				}
			break;
			}
			objekty[i]->UpdateObjDisplay();
		glPopMatrix();
	}

	SDL_GL_SwapBuffers();
}
