// OGLfunc.cpp: implementation of the COGLfunc class.
//
//////////////////////////////////////////////////////////////////////

#include "OGLfunc.h"
#include <stdio.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

COGLfunc::COGLfunc() {}

COGLfunc::~COGLfunc()
{ glDeleteLists(baseFont, 96); }

GLvoid COGLfunc::ReSizeGLScene(GLsizei width, GLsizei height)
{									// Resize And Initialize The GL Window
	if (height==0) height=1;		// Prevent A Divide By Zero By
	glViewport(0, 0, width, height);// Reset The Current Viewport

	glMatrixMode(GL_PROJECTION);	// Select The Projection Matrix
	glLoadIdentity();				// Reset The Projection Matrix

	// Calculate The Aspect Ratio Of The Window
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.01f,10000.0f);

	glMatrixMode(GL_MODELVIEW);		// Select The Modelview Matrix
	glLoadIdentity();				// Reset The Modelview Matrix
}

int COGLfunc::InitGL()				// All Setup For OpenGL Goes Here
{
	glShadeModel(GL_SMOOTH);		// Enable Smooth Shading
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);	// Black Background
	glClearDepth(1.0f);				// Depth Buffer Setup
	glDepthFunc(GL_LEQUAL);			// The Type Of Depth Testing To Do
	glEnable(GL_DEPTH_TEST);		// Enables Depth Testing
	glEnable(GL_COLOR_MATERIAL);	// Enable Material Coloring
	glEnable(GL_TEXTURE_2D);		// Enable Texture Mapping ( NEW )
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);	// Really Nice Perspective Calculations
	glHint(GL_POINT_SMOOTH_HINT,GL_NICEST);		// Really Nice Point Smoothing

	//lights
	static GLfloat LightAmbient[]= { 0.0f, 0.0f, 0.0f, 1.0f }; //vsude
	static GLfloat LightDiffuse[]= { 1.0f, 1.0f, 1.0f, 1.0f }; //ze zdroje
	static GLfloat LightPosition[]= { 0.0f, 0.0f, 1.0f, 1.0f };				 // Light Position 
	glLightfv(GL_LIGHT1, GL_AMBIENT, LightAmbient);		// Setup The Ambient Light
	glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse);		// Setup The Diffuse Light
	glLightfv(GL_LIGHT1, GL_POSITION,LightPosition);	// Position The Light
	glEnable(GL_LIGHT1);

	//blending
	glColor4f(1.0f, 1.0f, 1.0f, 0.5f);	// Full Brightness.  50% Alpha
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);	// Set The Blending Function For Translucency

	/*//fog
	static GLfloat fogColor[4]= {0.5f, 0.5f, 0.5f, 1.0f};
	glFogfv(GL_FOG_COLOR, fogColor);	// Set Fog Color
	glFogf(GL_FOG_DENSITY, 1.35f);		// How Dense Will The Fog Be
	glHint(GL_FOG_HINT, GL_DONT_CARE);	// Fog Hint Value, GL_NICEST or GL_FASTEST
	glFogf(GL_FOG_START, 0.0f);			// Fog Start Depth
	glFogf(GL_FOG_END, 30.0f);			// Fog End Depth
	fogging(0);
	*/

	BuildFont();						// Build The Font

	return 1;							// Initialization Went OK
}

void COGLfunc::fogging(int fog)
{
	switch (fog) {
	case 0: glDisable(GL_FOG); break;
	case 1:
		glFogi(GL_FOG_MODE, GL_EXP);
		glEnable(GL_FOG);
		break;
	case 2:
		glFogi(GL_FOG_MODE, GL_EXP2);
		glEnable(GL_FOG);
		break;
	case 3:
		glFogi(GL_FOG_MODE, GL_LINEAR);
		glEnable(GL_FOG);
		break;
	}
}

void COGLfunc::blending(int blend)
{
	if(blend) {
		glEnable(GL_BLEND);
		glDisable(GL_DEPTH_TEST);
	} else {
		glDisable(GL_BLEND);
		glEnable(GL_DEPTH_TEST);
	}
}

void COGLfunc::lighting(int light)
{
	if ( light) glEnable(GL_LIGHTING); else glDisable(GL_LIGHTING);
}

AUX_RGBImageRec *COGLfunc::LoadBMP(char *Filename)
{
	FILE *File=NULL;

	if (!Filename) return NULL;
	
	File=fopen(Filename,"r");		// Check To See If The File Exists
	if (!File) return NULL;
	fclose(File);

	return auxDIBImageLoad(Filename);	// Load The Bitmap And Return A Pointer
}

int COGLfunc::LoadGLTextures(GLuint texture,char *Filename, int kvalita)
{
	int Status=FALSE;							// Status Indicator
	AUX_RGBImageRec *TextureImage[1];			// Create Storage Space For The Texture
	memset(TextureImage,0,sizeof(void *)*1);	// Set The Pointer To NULL

	// Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit
	if ( TextureImage[0]=LoadBMP(Filename)) {
		Status=TRUE;

		glBindTexture(GL_TEXTURE_2D, texture);
		switch (kvalita) {
		case 0:// Create Nearest Filtered Texture | proc to nefunguje?
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
			glTexImage2D(GL_TEXTURE_2D, 0, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
		break;
		case 1:// Create Linear Filtered Texture | proc to nefunguje?
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
			glTexImage2D(GL_TEXTURE_2D, 0, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
		break;
		default:// Create MipMapped Texture
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
			gluBuild2DMipmaps(GL_TEXTURE_2D, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
		break;
		}
	}

	if (TextureImage[0]) {
		if (TextureImage[0]->data) free(TextureImage[0]->data);
		free(TextureImage[0]);
	}
	return Status;
}

////////////////////Pismo/////////////////////

void COGLfunc::BuildFont()					// Build Our Bitmap Font
{
	HFONT	oldfont,font;						// Windows Font ID
	HDC hDC=GetDC(NULL);

	baseFont = glGenLists(96);					// Storage For 96 Characters ( NEW )

	font = CreateFont(	-20,				// Height Of Font ( NEW )
				0,				// Width Of Font
				0,				// Angle Of Escapement
				0,				// Orientation Angle
				FW_BOLD,			// Font Weight
				FALSE,				// Italic
				FALSE,				// Underline
				FALSE,				// Strikeout
				ANSI_CHARSET,			// Character Set Identifier
				OUT_TT_PRECIS,			// Output Precision
				CLIP_DEFAULT_PRECIS,		// Clipping Precision
				ANTIALIASED_QUALITY,		// Output Quality
				FF_DONTCARE|DEFAULT_PITCH,	// Family And Pitch
				"Courier New");			// Font Name

	oldfont = (HFONT)SelectObject(hDC, font);		// Selects The Font We Want
	wglUseFontBitmaps(hDC, 32, 96, baseFont);			// Builds 96 Characters Starting At Character 32
	SelectObject(hDC, oldfont);				// Selects The Font We Want
	DeleteObject(font);					// Delete The Font
}

GLvoid COGLfunc::glPrint3d(float x, float y, float z, float r, float g, float b, const char *fmt, ...)				// Custom GL "Print" Routine
{
	glBindTexture(GL_TEXTURE_2D, 0);
	glColor3f(r,g,b);
	glRasterPos3f(x,y,z);
	
	char		text[256];			// Holds Our String
	va_list		ap;					// Pointer To List Of Arguments
	if (fmt == NULL)				// If There's No Text
		return;						// Do Nothing
	va_start(ap, fmt);				// Parses The String For Variables
	    vsprintf(text, fmt, ap);	// And Converts Symbols To Actual Numbers
	va_end(ap);						// Results Are Stored In Text
	glPushAttrib(GL_LIST_BIT);		// Pushes The Display List Bits		( NEW )
	glListBase(baseFont - 32);			// Sets The Base Character to 32	( NEW )
	glCallLists(strlen(text), GL_UNSIGNED_BYTE, text);	// Draws The Display List Text	( NEW )
	glPopAttrib();					// Pops The Display List Bits	( NEW )

	glColor3f(1,1,1);
}

GLvoid COGLfunc::glPrint2d(float x, float y, float r, float g, float b, const char *fmt, ...)				// Custom GL "Print" Routine
{
	char		text[256];			// Holds Our String
	va_list		ap;					// Pointer To List Of Arguments
	if (fmt == NULL)				// If There's No Text
		return;						// Do Nothing
	va_start(ap, fmt);				// Parses The String For Variables
	    vsprintf(text, fmt, ap);	// And Converts Symbols To Actual Numbers
	va_end(ap);						// Results Are Stored In Text

	glPrint3d(x*0.55f,y*0.41f,-1.001f,r,g,b,text);
}
